package ever.generic;

import java.util.HashMap;
import java.util.LinkedList;

import ever.pipeline.AbstractLocalWorkflowAnalyzer;
import ever.pipeline.Case;
import ever.workflowRepresentation.Sequence;
import ever.workflowRepresentation.Task;
import ever.workflowRepresentation.WorkflowElement;
/**
 * It is possible that different analyzers create the same task multiple times. This analyzer search for such double occurences and repairs it by merging 
 * the tasks to one task.
 * @author Pol Schumacher, Wirtschaftsinformatik, Institut fuer Informatik, Goethe Universitaet Frankfurt
 * 
 */
public class LocalSequenceTaskDuplicateRemover extends
		AbstractLocalWorkflowAnalyzer {

	@Override
	public Case analyze(Case c) {
		Sequence seq = c.getWf().getTopSequence();
		LinkedList<WorkflowElement> list = seq.getAllElements();
		if(list.size()==0)
			return c;
		
		LinkedList<WorkflowElement> out = new LinkedList<WorkflowElement>();
		for (int i=0; i<list.size()-1;i++) {			
			try {
				Task t = ((Task) list.get(i));
				Task p = ((Task) list.get(i+1)); 								
				if (p.getName().equals(t.getName()) & p.relatedSentence.getRaw().equals(t.relatedSentence.getRaw())) {								
					
					
					Task mergedTask = Task.merge(p,t);
					mergedTask.wasMerged=true;														
					list.set(i, mergedTask);
					list.remove(i+1);					
					int indexOfSen = c.getSentences().indexOf(t.relatedSentence);
					t.relatedSentence.removeRelWfElement(t);
					t.relatedSentence.removeRelWfElement(p);
					t.relatedSentence.addRelWfElement(mergedTask);					
					c.getSentences().set(indexOfSen, t.relatedSentence);				
					if(i+1==list.size())
						break;
					
					i--;			
					
					continue;
				} else {					
					out.add(t);														
					continue;
				}
			} catch (ClassCastException ex) {
				out.add(list.get(i));
				continue;
			} catch (Exception ex)
			{
				ex.printStackTrace();
			}
			
		}
		
		try {
			out.add(list.getLast());
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		seq.putAllElements(out);		
		c.getWf().setTopSequence(seq);
		return c;
	}

	@Override
	public HashMap<Integer, Case> analyze(HashMap<Integer, Case> c) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void analyze() {
		// TODO Auto-generated method stub

	}

}
