package ever.generic;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import ever.pipeline.AbstractLocalInputAnalyzer;
import ever.pipeline.Case;
import ever.pipeline.Clause;
import ever.pipeline.Sentence;
import ever.workflowRepresentation.EmptyProductException;
import ever.workflowRepresentation.NotAProductException;
import ever.workflowRepresentation.Product;
import ever.workflowRepresentation.Sequence;
import ever.workflowRepresentation.SimpleFacet;
import ever.workflowRepresentation.SimpleProduct;
import ever.workflowRepresentation.Task;
import ever.workflowRepresentation.Workflow;
import ever.workflowRepresentation.WorkflowElement;

/**
 * This class extracts tasks which look like cook for... so for which no case frame would be created
 * @author Pol Schumacher, Wirtschaftsinformatik, Institut fuer Informatik, Goethe Universitaet Frankfurt
 */
public class VerbAtBeginning extends AbstractLocalInputAnalyzer {

	@Override
	
	public Case analyze(Case c) {
		Workflow wf = c.getWf();
		Sequence seq = null;
		ArrayList<Task> taskList = new ArrayList<Task>();
		if (wf == null) {
			wf = new Workflow();
			seq = new Sequence(null, null);

		} else {
			seq = (Sequence) wf.getTopSequence();
		}
		ArrayList<Sentence> senList = c.getSentences();

		for (int i = 0; i < senList.size(); i++) {
			Sentence s = senList.get(i);
			if (s.getRaw().split(" ").length < 2)
				continue;

			if (s.getRaw().split(" ")[1].equals("and")) {
				Task t = null;
				Task first = null;
//				System.out.println(s.getRaw() + " " + c.getName());
				for (int j = 0; j < seq.getAllElements().size(); j++) {
					t = (Task) seq.getAllElements().get(j);
					if (t.getName() == null)
						continue;

					first = new Task(t.getParentElement(), s);					
					
					if (t.getName().equals(s.getRaw().split(" ")[2])) {
						first.setName(s.getRaw().split(" ")[0]);
						//System.out.println(s.getRaw().split(" ")[0]);
						for (Product p : t.getAllInProducts()) {
							first.addInProduct(p);
						}

					}
					else
						continue;

					
//					
					first.addRelSentence(s);
					seq.putElementAt(first, seq.indexOf(t));
					s.addRelWfElement(first);
					
					break;
				}

			}

		}

		c.setWf(wf);
		return c;
	}

	@Override
	public HashMap<Integer, Case> analyze(HashMap<Integer, Case> c) {
		// TODO Auto-generated method stub
		// throw new NotImplementedException();
		return null;

	}

	@Override
	public void analyze() {
		// throw new NotImplementedException();

	}

}
