package ever.simpleCooking;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;

import ever.pipeline.AbstractLocalInputAnalyzer;
import ever.pipeline.Case;
import ever.pipeline.CaseFrame;
import ever.pipeline.Clause;
import ever.pipeline.Sentence;
import ever.workflowRepresentation.And;
import ever.workflowRepresentation.EmptyProductException;
import ever.workflowRepresentation.NotAProductException;
import ever.workflowRepresentation.Option;
import ever.workflowRepresentation.Product;
import ever.workflowRepresentation.Sequence;
import ever.workflowRepresentation.SimpleProduct;
import ever.workflowRepresentation.Task;
import ever.workflowRepresentation.Workflow;
import ever.workflowRepresentation.WorkflowElement;
import ever.workflowRepresentation.Xor;
/**
 * This class applies several pattern to extract a disjunctiv control-flow
 * @author Pol Schumacher, Wirtschaftsinformatik, Institut fuer Informatik, Goethe Universitaet Frankfurt
 *
 */
public class LocalXorDetector extends AbstractLocalInputAnalyzer {

	private String raw;
	private ArrayList<CaseFrame> caseFrameList;
	private ArrayList<Clause> clauseList;
	private Sentence s;
	private Case cas;

	@Override
	public Case analyze(Case cas) {
		this.cas = cas;
		Workflow wf = cas.getWf();
		Sequence seq = null;
		ArrayList<WorkflowElement> taskList = new ArrayList<WorkflowElement>();
		if (wf == null) {
			wf = new Workflow();
			seq = new Sequence(null,null);
		} else {
			seq = (Sequence) wf.getTopSequence();
		}
		ArrayList<Sentence> senList = cas.getSentences();
		Xor a = null;
		for (int i = 0; i < senList.size(); i++) {
			caseFrameList = senList.get(i).getCaseFrames();
			clauseList = senList.get(i).getClauseList();
			raw = senList.get(i).getRaw();
			s = senList.get(i);

			if (checkVerb1OrVerb2()) {
				a = extractVerbOrVerb();
			} else {
				if (checkIfNecessaryBig()) {
					a = extractIfNecessaryBig();
				} else {
					if (checkIfNecessarySmall()) {
						a = extractIfNecessaryBig();
					} else {
						if (checkIfDesiredBig()) {
							a = extractIfDesiredBig();
						} else {
							if (checkIfDesiredSmall()) {
								a = extractIfDesiredBig();
							} else
							{
								if(checkAlternativelyWith())
								{
									a = extractAlternativelyWith();
								}
								else
									continue;
							}
								
						}
					}
				}
			}

			int pos = super.getNextRelated(senList, s, seq);					
			try {
				seq.putElementAt(a, pos);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}

		wf.setTopSequence(seq);
		cas.setWf(wf);
		return cas;
	}

	private Xor extractAlternativelyWith() {
		Xor x = new Xor(cas.getWf().getTopSequence(),s);
		boolean found=false;
		for(int i=0; i<s.getCaseFrames().size(); i++)
		{
			CaseFrame current = s.getCaseFrames().get(i);
			if(current.getPpType()==null)
				continue;
			
			found=true;
			if(current.getPpType().equals("WITH"))
			{
				Sequence seq = new Sequence(x,s);
				Task t = new Task(seq,s);
				t.setName(current.getTrigger());
				try {
					t.addInProduct(new SimpleProduct(current.getEx(),t.getId()));
				} catch (EmptyProductException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (NotAProductException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}				
				seq.addWfElement(t);				
				x.addOption(new Option(seq, ""));
				if(i>0)
				{
					CaseFrame prev = s.getCaseFrames().get(i-1);
					if(prev.getTrigger().equals(current.getTrigger()))
					{
						Sequence seq2 = new Sequence(x,s);
						Task t2 = new Task(seq2,s,prev.getTrigger().toLowerCase());						
						seq2.addWfElement(t2);						
						x.addOption(new Option(seq2, ""));												
					}
				}
				break;
			}						
		}
		for(WorkflowElement wfe : s.getRelatedElem())
		{
			cas.getWf().getTopSequence().removeWfe(wfe);
		}
		s.getRelatedElem().clear();		
		s.addRelWfElement(x);
		x.addRelSentence(s);
		return x;		
	}

	@Override
	public HashMap<Integer, Case> analyze(HashMap<Integer, Case> c) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void analyze() {
		// TODO Auto-generated method stub

	}

	/**
	 * Checks if any xor type was found
	 * 
	 * @return true if an xor-type was found
	 */
	public boolean isXor() {
		if (checkIfDesiredSmall() | checkIfDesiredBig()
				| checkIfNecessarySmall() | checkIfNecessaryBig()
				| checkVerb1OrVerb2()) {
			return true;
		}
		return false;
	}

	/**
	 * Should locate And flows for cases like :"Verb1 or Verb2 the dobj
	 * 
	 * @return true if the pattern was found
	 */
	public boolean checkVerb1OrVerb2() {
		if (raw.contains(" or ")) {
			boolean verb1 = false, verb2 = false, or = false;
			int counter = 0, posVerb1 = 0, posVerb2 = 0, posOr = 0;
			Iterator<Clause> it = clauseList.iterator();
			while (it.hasNext()) {
				Clause c = it.next();
				if (c.getWholeTyp().contains("VP SEGMENT (ACTIVE_VERB):")
						& !verb1 & !verb2 & !or) {
					verb1 = true;
					posVerb1 = counter;
				}

				if (c.getInfos().get(0).equals("(LEX)(CONJ)")
						& c.getValue().equals("or ") & verb1 & !verb2) {
					or = true;
					posOr = counter;
				}

				if (c.getWholeTyp().contains("VP SEGMENT (ACTIVE_VERB):") & or
						& verb1) {
					verb2 = true;
					posVerb2 = counter;
				}
				counter++;

				if (verb1 & verb2 & or & posOr - posVerb1 == 1
						& posVerb2 - posOr == 1) {
					return true;
				}

			}

		}
		return false;
	}

	/**
	 * Should locate "Add flour mixture alternately with zucchini into the egg mixture."
	 * 
	 * @return true if the pattern was found
	 */
	private boolean checkAlternativelyWith() {
		if (raw.contains(" alternately with")) {
			return true;
		}

		return false;
	}
	
	/**
	 * Should locate And flows for cases like :"Add cheese , if desired.
	 * 
	 * @return true if the pattern was found
	 */
	private boolean checkIfDesiredSmall() {
		if (raw.contains("if desired")) {
			return true;
		}

		return false;
	}

	/**
	 * Locates the xor pattern : If desired then do sth.
	 * 
	 * @return true if the pattern was found
	 */
	private boolean checkIfDesiredBig() {
		if (raw.contains("If desired")) {
			return true;
		}

		return false;
	}

	/**
	 * Locates the xor pattern : Do sth. if necessary
	 * 
	 * @return true if the pattern was found
	 */
	private boolean checkIfNecessarySmall() {
		if (raw.contains("if necessary")) {
			return true;
		}

		return false;
	}

	/**
	 * Locates the xor pattern : If necessary do sth
	 * 
	 * @return true if the pattern was found
	 */
	private boolean checkIfNecessaryBig() {
		if (raw.contains("If necessary")) {
			return true;
		}

		return false;
	}

	private Xor extractVerbOrVerb() {
		Xor xor = new Xor(cas.getWf().getTopSequence(),s);
		ArrayList<WorkflowElement> tl = s.getRelatedElem();
		boolean verb1 = false, verb2 = false, or = false;
		int counter = 0, posVerb1 = 0, posVerb2 = 0, posOr = 0;
		Iterator<Clause> it = clauseList.iterator();
		Task t1 = new Task(xor,s), t2 = new Task(xor,s);
		Sequence s1 = new Sequence(xor,s);
		Sequence s2 = new Sequence(xor,s);
		while (it.hasNext()) {
			Clause c = it.next();
			if (c.getWholeTyp().contains("VP SEGMENT (ACTIVE_VERB):") & !verb1
					& !verb2 & !or) {
				verb1 = true;
				t1.setName(c.getValue());
				posVerb1 = counter;
			}

			if (c.getInfos().get(0).equals("(LEX)(CONJ)")
					& c.getValue().equals("or ") & verb1 & !verb2) {
				or = true;
				posOr = counter;
			}

			if (c.getWholeTyp().contains("VP SEGMENT (ACTIVE_VERB):") & or
					& verb1) {
				verb2 = true;
				String name = c.getValue();
				for (WorkflowElement wfe : s.getRelatedElem()) {
					try {
						Task t = (Task) wfe;
						if (t.getName().equals(name))
							;
						{
							t2 = t;
							Product p = t2.getNextInProduct();
							HashSet<Product> ps=new HashSet<Product>();
							while(p!=null)
							{
								ps.add(p);
								p=t2.getNextInProduct();
							}
							
							for(Product pt: ps)
							{
								t2.addInProduct(pt);
								t.addInProduct(pt);
							}
						}

					} catch (ClassCastException ex) {
						continue;
					}
				}				
			}								
		}
		s1.addWfElement(t1);
		s1.addRelSentence(s);
		s2.addWfElement(t2);
		s2.addRelSentence(s);
		s.addRelWfElement(xor);
		xor.addOption(new Option(s1,""));
		xor.addOption(new Option(s2,""));

		return xor;
	}

	private Xor extractIfDesiredBig() {
		Xor a = new Xor(cas.getWf().getTopSequence(),s);
		ArrayList<WorkflowElement> tl = s.getRelatedElem();
		Sequence seq = new Sequence(a,s);
		/*
		 * Take direct following task
		 */
		for (WorkflowElement t : tl) {
			seq.addWfElement(t);
			cas.getWf().getTopSequence().removeWfe(t);
		}
		a.addOption(new Option(seq, "If desired"));
		a.addRelSentence(s);
		s.addRelWfElement(a);
		return a;
	}

//	private Xor extractIfDesiredSmall() {
//		Xor a = new Xor(cas.getWf().getTopSequence());
//		ArrayList<WorkflowElement> tl = s.getRelatedElem();
//
//		Sequence seq = new Sequence(a);
//		/*
//		 * Take direct predecent task, we imply that the if desired is linked to
//		 * this
//		 */
//		int indexIf = raw.indexOf("if desired");
//		for (int i = 0; i < tl.size(); i++) {
//			try {
//				if (raw.indexOf(((Task) tl.get(i)).getName().toLowerCase()) < indexIf) {
//					tl.remove(i);
//					i--;
//				}
//			} catch (ClassCastException ex) {
//				continue;
//			}
//
//		}
//
//		for (WorkflowElement t : tl) {
//			cas.getWf().getTopSequence().removeWfe(t);
//			t.addRelSentence(s);
//			seq.addRelSentence(s);
//			seq.addWfElement(t);
//			s.removeRelWfElement(t);
//		}
//		s.addRelWfElement(a);
//		a.addRelSentence(s);
//		return a;
//	}
//
//	private Xor extractIfNecessarySmall() {
//		Xor a = new Xor(cas.getWf().getTopSequence());
//		ArrayList<WorkflowElement> tl = s.getRelatedElem();
//
//		Sequence seq = new Sequence(a);
//		/*
//		 * Take direct predecent task, we imply that the if desired is linked to
//		 * this
//		 */
//		int indexIf = raw.indexOf("if necessary");
//		if(indexIf==0)
//			return extractIfNecessaryBig();
//		for (int i = 0; i < tl.size(); i++) {
//			try {
//				if (raw.indexOf(((Task) tl.get(i)).getName().toLowerCase()) < indexIf) {
//					tl.remove(i);
//					i--;
//				}
//			} catch (ClassCastException ex) {
//				continue;
//			}
//
//		}
//
//		for (int i=0; i<tl.size(); i++) {			
//			cas.getWf().getTopSequence().removeWfe(tl.get(i));
//			tl.get(i).addRelSentence(s);
//			seq.addRelSentence(s);
//			seq.addWfElement(tl.get(i));
//			s.removeRelWfElement(tl.get(i));
//		}
//		s.addRelWfElement(a);
//		a.addRelSentence(s);
//		return a;
//	}

	private Xor extractIfNecessaryBig() {
		Xor a = new Xor(cas.getWf().getTopSequence(),s);
		ArrayList<WorkflowElement> tl = s.getRelatedElem();
		Sequence seq = new Sequence(a,s);
		/*
		 * Take direct following task
		 */
		for (WorkflowElement t : tl) {
			seq.addWfElement(t);
			cas.getWf().getTopSequence().removeWfe(t);
		}
		for (int i=0; i<tl.size(); i++) {			
			cas.getWf().getTopSequence().removeWfe(tl.get(i));
			tl.get(i).addRelSentence(s);
			seq.addRelSentence(s);
			seq.addWfElement(tl.get(i));
			s.removeRelWfElement(tl.get(i));
		}
		s.addRelWfElement(a);
		a.addRelSentence(s);
		a.addOption(new Option(seq, "If necessary"));

		return a;
	}

}
