package ever.workflowRepresentation;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;
/**
 * This class is used to represent additional information. This information can be ether linked to a task or to a product.
 * The information must be representated in description-value pair. 
 * @author Pol Schumacher, Wirtschaftsinformatik, Institut fuer Informatik, Goethe Universitaet Frankfurt
 *
 */
public class SimpleFacet extends Facet{
	/**
	 * Describes the semantics of a value
	 */
	private String facetDefinition;
	/**
	 * The value of the facet
	 */
	private String facetValue;
	
	/**
	 * Sets value and definition
	 * @param facetDef
	 * @param facetVal
	 */
	public SimpleFacet(String facetDef, String facetVal)
	{
		facetDefinition=facetDef;
		facetValue=facetVal;
	}
	/**
	 * Returns the definition of the facet
	 * @return definition
	 */
	public String getFacetDefinition() {
		return facetDefinition;
	}
	/**
	 * Sets the definition of the facet.
	 * @param facetDefinition
	 */
	public void setFacetDefinition(String facetDefinition) {
		this.facetDefinition = facetDefinition;
	}
	/**
	 * Gets facet value.
	 * @return value
	 */
	public String getFacetValue() {
		return facetValue;
	}
	/**
	 * Sets facet value.
	 * @param facetValue
	 */
	public void setFacetValue(String facetValue) {
		this.facetValue = facetValue;
	}
	/**
	 * Writes facets to output-format
	 * @param writer stream-writer that is used to write out
	 */
	public void exportWf2Xml(XMLStreamWriter writer) throws XMLStreamException
	{		
		writer.writeEmptyElement("cdol", "AA",
				"http://cake.wi2.uni-trier.de/xml/cdol");
		writer.writeAttribute("n", getFacetDefinition());
		writer.writeAttribute("v", getFacetValue());
		writer.writeCharacters("\n");
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((facetDefinition == null) ? 0 : facetDefinition.hashCode());
		result = prime * result
				+ ((facetValue == null) ? 0 : facetValue.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		SimpleFacet other = (SimpleFacet) obj;
		if (facetDefinition == null) {
			if (other.facetDefinition != null)
				return false;
		} else if (!facetDefinition.equals(other.facetDefinition))
			return false;
		if (facetValue == null) {
			if (other.facetValue != null)
				return false;
		} else if (!facetValue.equals(other.facetValue))
			return false;
		return true;
	}
	
	

}
